@@ -9,7 +9,7 @@ Exports an error box that can be used to pass the errors we need.
 
 Further exports helper functions for creating errors
 of different flavors. We wrap these functions in the same
-EditError box to avoid creating too many new classes!
+MitoError box to avoid creating too many new classes!
 
 See more about why we use errors here: 
 https://stackoverflow.com/questions/16138232/is-it-a-good-practice-to-use-try-except-else-in-python
@@ -17,28 +17,12 @@ https://stackoverflow.com/questions/16138232/is-it-a-good-practice-to-use-try-ex
 import traceback
 from typing import Set, List
 
-class CreationError(Exception):
-    """
-    An error that occurs during a mito.sheet call, on creation
-    of the mito widget. 
-    """
-    def __init__(self, type_, header, to_fix):
-        """
-        Creates a creation error. 
 
-        type_: the error id string.
-        header: the header of the error message to be displayed.
-        to_fix: instructions on how to handle or fix the error.
-        """
-        self.type_ = type_ # we have an _ to avoid overwriting the build in type
-        self.header = header
-        self.to_fix = to_fix
-
-class EditError(Exception):
+class MitoError(Exception):
     """
     An error that occurs during the processing of an editing event.
     """
-    def __init__(self, type_, header, to_fix):
+    def __init__(self, type_, header, to_fix, error_modal=True):
         """
         Creates a creation error. 
 
@@ -49,6 +33,7 @@ class EditError(Exception):
         self.type_ = type_ # we have an _ to avoid overwriting the build in type
         self.header = header
         self.to_fix = to_fix
+        self.error_modal = error_modal
 
 def make_no_sheet_error(sheet_indexes: Set[str]):
     """
@@ -58,13 +43,28 @@ def make_no_sheet_error(sheet_indexes: Set[str]):
     """
     to_fix = f'There is an issue behind the scenes with that operation, we\'ll get it fixed as soon as possible!'
 
-    return EditError(
+    return MitoError(
         'no_sheet_error', 
         'No Sheet Exists',
         to_fix
     )
 
-def make_no_column_error(column_headers: Set[str]):
+def make_incompatible_merge_key_error(error_modal=True):
+    """
+    Helper function for creating a incompatible_merge_key_error
+
+    Occurs when a user merges on two keys that are different types
+    """
+    to_fix = f'You cannot merge using keys of different types. Either pick new keys or cast their types.'
+
+    return MitoError(
+        'incompatible_merge_key_error', 
+        'Incompatible Merge Key Types',
+        to_fix,
+        error_modal=error_modal
+    )
+
+def make_no_column_error(column_headers: Set[str], error_modal=True):
     """
     Helper function for creating a no_column_error.
 
@@ -75,10 +75,11 @@ def make_no_column_error(column_headers: Set[str]):
     else:
         to_fix = f'Sorry, there are no column with the names {", ".join(column_headers)}. Did you type them correctly?'
 
-    return EditError(
+    return MitoError(
         'no_column_error', 
         'No Column Exists',
-        to_fix
+        to_fix,
+        error_modal=error_modal
     )
 
 def make_column_exists_error(column_header: str):
@@ -88,13 +89,13 @@ def make_column_exists_error(column_header: str):
     Occurs when:
     -  the user adds a column that already exists in the dataframe.
     """
-    return EditError(
+    return MitoError(
         'column_exists_error', 
         'Column Already Exists',
         f'Sorry, a column already exists with the name {column_header}. Try picking a different name!'
     )
 
-def make_invalid_formula_error(formula: str, to_fix=None):
+def make_invalid_formula_error(formula: str, to_fix=None, error_modal=True):
     """
     Helper function for creating a invalid_formula_error.
 
@@ -104,36 +105,55 @@ def make_invalid_formula_error(formula: str, to_fix=None):
     if to_fix is None:
         to_fix = f'Sorry, the formula \'{formula}\' is invalid. Did you type it correctly?'
 
-    return EditError(
+    return MitoError(
         'invalid_formula_error',
         'Invalid Formula',
-        to_fix
+        to_fix, 
+        error_modal=error_modal
     )
 
-def make_circular_reference_error():
+
+def make_cast_value_to_type_error(value: str, target_type: str, error_modal=True):
+    """
+    Helper function for creating a cast_value_to_type_error.
+
+    Occurs when the user edits a cell to a specific value that cannot be cast to the 
+    type of the column it is in.
+
+    """
+    return MitoError(
+        'cast_value_to_type_error',
+        'Invalid Value for ' + target_type + 'Column',
+        'The value ' + value + ' could not be cast to the type ' + target_type + '. Please enter a different value.' , 
+        error_modal=error_modal
+    )
+
+def make_circular_reference_error(error_modal=True):
     """
     Helper function for creating a circular_reference_error.
 
     Occurs when:
     -  an edit introduces a circular reference.
     """
-    return EditError(
+    return MitoError(
         'circular_reference_error',
         'Circular Reference',
-        f'Sorry, circular references are not supported currently.'
+        f'Sorry, circular references are not supported currently.',
+        error_modal=error_modal
     )
 
-def make_wrong_column_metatype_error(column_header: str):
+def make_wrong_column_metatype_error(column_header: str, error_modal=True):
     """
     Helper function for creating a wrong_column_metatype_error.
 
     Occurs when:
     -  a user edits a column to a formula when it is not a formula type column.
     """
-    return EditError(
+    return MitoError(
         'wrong_column_metatype_error',
         'Wrong Column Type',
-        f'Sorry, the column {column_header} is a data column. We don\'t currently support changing it to a formula.'
+        f'Sorry, the column {column_header} is a data column. We don\'t currently support changing it to a formula.',
+        error_modal=error_modal
     )
 
 def make_invalid_column_headers_error(column_headers: List[str]):
@@ -145,13 +165,29 @@ def make_invalid_column_headers_error(column_headers: List[str]):
     """
     to_fix = f'All headers in the dataframe must contain at least one letter and no symbols other than numbers and "_". Invalid headers: {", ".join(column_headers)}'
 
-    return EditError(
+    return MitoError(
         'invalid_column_header_error',
         'Invalid Column Header',
         to_fix
     )
 
-def make_execution_error():
+def make_function_error(function_name, error_modal=True):
+    """
+    Helper function for creating a function_error.
+
+    Occurs when there is an error inside a specific sheet function that
+    is running
+    """
+    # TODO: this to_fix message is _just terrible_
+    return MitoError(
+        'function_error',
+        'Error in ' + function_name,
+        f'Sorry, an unknown error occured in the ' + function_name + ' function. See the documentation for help.',
+        error_modal=error_modal
+    )
+
+
+def make_execution_error(error_modal=True):
     """
     Helper function for creating a execution_error.
 
@@ -159,10 +195,11 @@ def make_execution_error():
     -  running an existing analysis on the parameters passed to the mito sheet throw an error.
     """
     # TODO: this to_fix message is _just terrible_
-    return EditError(
+    return MitoError(
         'execution_error',
         'Execution Error',
-        f'Sorry, there was an error during executing this code.'
+        f'Sorry, there was an error during executing this code.',
+        error_modal=error_modal
     )
 
 def make_function_execution_error(function):
@@ -173,13 +210,13 @@ def make_function_execution_error(function):
     -  An error occurs inside of a mito sheet function
     """
     # TODO: this to_fix message is _just terrible_
-    return EditError(
+    return MitoError(
         'execution_error',
         f'Error Executing {function}',
         f'Sorry, there was an error in the {function}. Please check the documentation to make sure you called it correctly.'
     )
 
-def make_unsupported_function_error(functions: Set[str]):
+def make_unsupported_function_error(functions: Set[str], error_modal=True):
     """
     Helper function for creating a unsupported_function_error.
 
@@ -192,10 +229,11 @@ def make_unsupported_function_error(functions: Set[str]):
     else:
         to_fix = f'Sorry, mito does not currently support the functions {", ".join(functions)}. To request it, shoot us an email!'
 
-    return EditError(
+    return MitoError(
         'unsupported_function_error',
         'Unsupported Function',
-        to_fix
+        to_fix,
+        error_modal=error_modal
     )
 
 def make_invalid_column_delete_error(column_header, dependents):
@@ -216,7 +254,7 @@ def make_invalid_column_delete_error(column_header, dependents):
     else:
         dependent_string = f'columns {", ".join(dependents[: -1])}, and {dependents[-1]}'
 
-    return EditError(
+    return MitoError(
         'invalid_column_delete_error',
         'Column Has Dependents',
         f'{column_header} cannot be deleted, as it is referenced in {dependent_string}. Please remove these references before deleting.'
@@ -229,7 +267,7 @@ def make_invalid_arguments_error(function):
     Occurs when:
     -  the user calls a sheet function with incorrect arguments
     """
-    return EditError(
+    return MitoError(
         'invalid_arguments_error',
         f'Invalid Arguments to {function}',
         f'It looks like you passed the wrong arguments to {function}. Try checking out the documentation to see correct usage!'
@@ -243,7 +281,7 @@ def make_invalid_aggregation_error():
     -  the user tries to .agg with a function on a column that has the wrong type (e.g. 'mean' on a
         column of strings).
     """
-    return EditError(
+    return MitoError(
         'invalid_aggregation_error',
         f'Invalid Aggregation Function',
         f'Sorry, you tried to aggregate with a function that could not be applied to that type of column. Please try again!'
@@ -265,7 +303,7 @@ def make_invalid_filter_error(filter_value, correct_type):
     else:
         correct_format = 'that is valid.'
 
-    return EditError(
+    return MitoError(
         'invalid_filter_error',
         f'Invalid Filter',
         f'Sorry, the value {filter_value} is not a valid value for that {correct_type} filter. Please enter a value {correct_format}!'
@@ -280,7 +318,7 @@ def make_invalid_sort_error(column_header):
     (e.g. a column with strings and floats).
     """
 
-    return EditError(
+    return MitoError(
         'invalid_sort_error',
         f'Invalid Sort',
         f'Sorry, the column {column_header} has mixed data types. Please make sure the column has one datatype before trying to sort.'
@@ -295,7 +333,7 @@ def make_df_exists_error(df_name):
     dataframe in the sheet.
     """
 
-    return EditError(
+    return MitoError(
         'df_exists_error',
         f'Dataframe Exists',
         f'Sorry, the dataframe {df_name} already exists. Please pick a different name.'
@@ -310,7 +348,7 @@ def make_invalid_column_type_change_error(column_header, old_dtype, new_dtype):
       a number to a datetime, which is impossible).
     """
 
-    return EditError(
+    return MitoError(
         'invalid_column_type_change_error',
         f'Invalid Type Change',
         f'Sorry, the column {column_header} has a type {old_dtype}, which cannot be changed to the type {new_dtype}.'
@@ -324,7 +362,7 @@ def make_invalid_pivot_error():
     -  the user runs a pivot that is invalid in some way.
     """
     # TODO: this to_fix message is _just terrible_
-    return EditError(
+    return MitoError(
         'invalid_pivot_error',
         'Pivot Error',
         f'Sorry, there was an error computing your pivot. Please try a different pivot!'
@@ -338,7 +376,7 @@ def make_is_directory_error(file_name):
     Occurs when:
     - the user tries to import a directory in a simple import
     """
-    return EditError(
+    return MitoError(
         'directory_error',
         f'Change Path to a File',
         f'Sorry, the path {file_name} is a directory. Please pass in a path to a file to import it into Mito.'
@@ -373,7 +411,7 @@ def make_operator_type_error(operator, arg_one_type, arg_two_type):
         # TODO: once we have a step that changes the column type, we should update this error to be more helpful
         to_fix = f'Sorry, you tried to use {operator} on a {type_one_full_name} and a {type_two_full_name}. Please change the type of one of these columns to be compatible.'
 
-    return EditError(
+    return MitoError(
         'operator_type_error',
         f'Error with {operator}',
         to_fix
