@@ -6,20 +6,16 @@
 
 
 from copy import copy
+from mitosheet.column_headers import flatten_column_header
+from typing import Any, Dict, List, Optional
+
+import pandas as pd
+from mitosheet.errors import (make_invalid_aggregation_error,
+                              make_invalid_pivot_error, make_no_column_error,
+                              make_no_sheet_error)
 from mitosheet.state import DATAFRAME_SOURCE_PIVOTED, State
 from mitosheet.step_performers.step_performer import StepPerformer
-from typing import Any, Dict, List, Optional
-from mitosheet.utils import make_valid_header
-import pandas as pd
 from pandas.core.base import DataError
-import pandas as pd
-
-from mitosheet.errors import (
-    make_invalid_pivot_error,
-    make_no_sheet_error,
-    make_no_column_error,
-    make_invalid_aggregation_error,
-)
 
 # Aggregation types pivot supports
 PA_COUNT_UNIQUE = 'count unique'
@@ -44,7 +40,7 @@ class PivotStepPerformer(StepPerformer):
 
     @classmethod
     def step_version(cls) -> int:
-        return 3
+        return 5
 
     @classmethod
     def step_type(cls) -> str:
@@ -68,16 +64,16 @@ class PivotStepPerformer(StepPerformer):
         result in errors without adding any data to the pivot.
         """
         # Case 1 - we have at least one row and at least one value
-        created_case_1 = len(params['pivot_rows']) > 0 and len(params['values']) > 0
+        created_case_1 = len(params['pivot_rows_column_ids']) > 0 and len(params['values_column_ids_map']) > 0
         # Case 2 - we have at least one column and at least one value
-        created_case_2 = len(params['pivot_columns']) > 0 and len(params['values']) > 0
+        created_case_2 = len(params['pivot_columns_column_ids']) > 0 and len(params['values_column_ids_map']) > 0
         params['created_non_empty_dataframe'] = created_case_1 or created_case_2
 
         # Filter out any duplicate aggregation functions
-        for column_header, aggregation_function_names in params['values'].items():
+        for column_id, aggregation_function_names in params['values_column_ids_map'].items():
             seen = set()
             aggregation_function_names = [x for x in aggregation_function_names if not (x in seen or seen.add(x))]
-            params['values'][column_header] = aggregation_function_names
+            params['values_column_ids_map'][column_id] = aggregation_function_names
 
         return params
 
@@ -86,10 +82,11 @@ class PivotStepPerformer(StepPerformer):
         cls,
         prev_state: State,
         sheet_index: int,
-        pivot_rows: List[str],
-        pivot_columns: List[str],
-        values: Dict[str, List[str]],
+        pivot_rows_column_ids: List[str],
+        pivot_columns_column_ids: List[str],
+        values_column_ids_map: Dict[str, List[str]],
         destination_sheet_index: int=None,
+        use_deprecated_id_algorithm: bool=False,
         **params
     ) -> State:
 
@@ -97,9 +94,16 @@ class PivotStepPerformer(StepPerformer):
         if not prev_state.does_sheet_index_exist_within_state(sheet_index):
             raise make_no_sheet_error(sheet_index)
 
+        pivot_rows = prev_state.column_ids.get_column_headers_by_ids(sheet_index, pivot_rows_column_ids)
+        pivot_columns = prev_state.column_ids.get_column_headers_by_ids(sheet_index, pivot_columns_column_ids)
+        values = {
+            prev_state.column_ids.get_column_header_by_id(sheet_index, column_id): value 
+            for column_id, value in values_column_ids_map.items()
+        }
+
         # We check that the pivot by doesn't use any columns that don't exist
         columns_used = set(pivot_rows).union(set(pivot_columns)).union(set(values.keys()))
-        missing_pivot_keys = columns_used.difference(prev_state.column_metatype[sheet_index].keys())
+        missing_pivot_keys = columns_used.difference(prev_state.dfs[sheet_index].keys())
         if len(missing_pivot_keys) > 0:
             raise make_no_column_error(missing_pivot_keys)
 
@@ -118,7 +122,8 @@ class PivotStepPerformer(StepPerformer):
             destination_sheet_index = post_state.add_df_to_state(
                 new_df, 
                 DATAFRAME_SOURCE_PIVOTED,
-                sheet_index=destination_sheet_index
+                sheet_index=destination_sheet_index,
+                use_deprecated_id_algorithm=use_deprecated_id_algorithm
             )
 
         except DataError as e:
@@ -140,10 +145,11 @@ class PivotStepPerformer(StepPerformer):
         post_state: State,
         execution_data: Optional[Dict[str, Any]],
         sheet_index: int,
-        pivot_rows: List[str],
-        pivot_columns: List[str],
-        values: Dict[str, List[str]],
+        pivot_rows_column_ids: List[str],
+        pivot_columns_column_ids: List[str],
+        values_column_ids_map: Dict[str, List[str]],
         destination_sheet_index: int=None,
+        use_deprecated_id_algorithm: bool=False,
         **params
     ) -> List[str]:
         old_df_name = post_state.df_names[sheet_index]
@@ -154,6 +160,13 @@ class PivotStepPerformer(StepPerformer):
             # to make sure to overwrite the correct pivot table 
             # by using the right name
             new_df_name = post_state.df_names[destination_sheet_index]
+
+        pivot_rows = prev_state.column_ids.get_column_headers_by_ids(sheet_index, pivot_rows_column_ids)
+        pivot_columns = prev_state.column_ids.get_column_headers_by_ids(sheet_index, pivot_columns_column_ids)
+        values = {
+            prev_state.column_ids.get_column_header_by_id(sheet_index, column_id): value 
+            for column_id, value in values_column_ids_map.items()
+        }
         
         # If there are no keys or values to aggregate on we return an empty dataframe. 
         if len(pivot_rows) == 0 and len(pivot_columns) == 0 or len(values) == 0:
@@ -176,7 +189,7 @@ class PivotStepPerformer(StepPerformer):
 
         # Finially, reset the column name, and the indexes!
         transpiled_code.append(RESET_INDEX_COMMENT)
-        transpiled_code.append(f'{new_df_name} = pivot_table.rename_axis(None, axis=1).reset_index()')
+        transpiled_code.append(f'{new_df_name} = pivot_table.reset_index()')
 
         return transpiled_code
 
@@ -184,10 +197,11 @@ class PivotStepPerformer(StepPerformer):
     def describe(
         cls,
         sheet_index,
-        pivot_rows,
-        pivot_columns,
-        values,
+        pivot_rows_column_ids,
+        pivot_columns_column_ids,
+        values_column_ids_map,
         destination_sheet_index=None,
+        use_deprecated_id_algorithm: bool=False,
         df_names=None,
         **params
     ) -> str:
@@ -261,10 +275,10 @@ def _execute_pivot(
     pivot_table = df.pivot_table(**args) # type: pd.DataFrame
 
     # Flatten the column headers
-    pivot_table.columns = [make_valid_header(col) for col in pivot_table.columns.values]
+    pivot_table.columns = [flatten_column_header(col) for col in pivot_table.columns.values]
 
-    # flatten the column headers & reset the indexes
-    pivot_table = pivot_table.rename_axis(None, axis=1).reset_index()
+    # Reset the indexes of the pivot table
+    pivot_table = pivot_table.reset_index()
 
     return pivot_table
 
@@ -284,7 +298,7 @@ TAB = '    '
 NEWLINE_TAB = f'\n{TAB}'
 
 FLATTEN_COMMENT = '# Flatten the column headers'
-FLATTEN_CODE = f'pivot_table.columns = [make_valid_header(col) for col in pivot_table.columns.values]'
+FLATTEN_CODE = f'pivot_table.columns = [flatten_column_header(col) for col in pivot_table.columns.values]'
 RESET_INDEX_COMMENT = '# Reset the column name and the indexes'
 
 
@@ -310,4 +324,4 @@ def build_args_code(
         args.append(f'values={values_keys},')
         args.append(f'aggfunc={values_to_functions_code(values)}')
         
-    return NEWLINE_TAB.join(args)
\ No newline at end of file
+    return NEWLINE_TAB.join(args)
