@@ -8,6 +8,7 @@ import uuid
 from copy import copy, deepcopy
 import pandas as pd
 from typing import List, Tuple
+from mitosheet.saved_analyses.save_utils import read_saved_analysis_names
 
 from mitosheet.step_performers.import_steps.simple_import import SimpleImportStepPerformer
 from mitosheet.state import State
@@ -186,33 +187,16 @@ class StepsManager():
         """
         return self.steps[self.curr_step_idx]
 
-    @property
-    def num_sheets(self) -> int:
-        """
-        Duh. :)
-        """
-        return len(self.steps[self.curr_step_idx].dfs)
-
     @property
     def dfs(self) -> List[pd.DataFrame]:
         return self.steps[self.curr_step_idx].dfs
 
     @property
-    def df_names_json(self) -> str:
-        """
-        A JSON object containing the names of the dataframes
-        """
-        return json.dumps({'df_names': self.curr_step.df_names})
-    
-    @property
-    def df_sources_json(self) -> str:
-        """
-        A JSON object containing the names of the dataframes
-        """
-        return json.dumps(self.curr_step.df_sources)
+    def data_type_in_mito(self) -> DataTypeInMito:
+        return get_data_type_in_mito(self.dfs)
 
     @property
-    def sheet_json(self) -> str:
+    def sheet_data_json(self) -> str:
         """
         sheet_json contains a serialized representation of the data
         frames that is then fed into the ag-grid in the front-end. 
@@ -221,99 +205,37 @@ class StepsManager():
         for speed reasons. This results in way less data getting 
         passed around
         """
-        return dfs_to_json(self.curr_step.dfs, self.curr_step.column_ids)
-    
-    @property
-    def df_shape_json(self) -> str:
-        """
-        Returns the df shape (rows, columns) of each dataframe in the 
-        current step!
-        """
-        return json.dumps([
-            {'rows': df.shape[0], 'cols': df.shape[1]}
-            for df in self.curr_step.dfs
-        ])
+        return dfs_to_json(
+            self.curr_step.dfs,
+            self.curr_step.df_names,
+            self.curr_step.df_sources,
+            self.curr_step.column_spreadsheet_code,
+            self.curr_step.column_filters,
+            self.curr_step.column_type,
+            self.curr_step.column_ids
+        )
 
     @property
-    def column_ids_array_json(self) -> str:
-        """
-        Returns a list of mappings from column ids -> column headers,
-        for each sheet.
-
-        We take special care to return the column ids array in the same order  
-        as the columns in the dataframe itself. 
-        """
-        column_ids_array = []
-        for sheet_index, df in enumerate(self.curr_step.dfs):
-            column_id_to_column_header_map = {
-                self.curr_step.column_ids.get_column_id_by_header(sheet_index, column_header): column_header
-                for column_header in df.keys()
-            }
-            column_ids_array.append(column_id_to_column_header_map)
-
-        return json.dumps(column_ids_array)
+    def analysis_data_json(self):
+        return json.dumps({
+            'analysisName': self.analysis_name,
+            'code': transpile(self),
+            'stepSummaryList': self.step_summary_list,
+            'currStepIdx': self.curr_step_idx,
+            'dataTypeInTool': self.data_type_in_mito.value
+        })
 
     @property
-    def column_spreadsheet_code_json(self) -> str:
-        """
-        column_spreadsheet_code_json is a list of all the spreadsheet
-        formulas that users have used, for each sheet they have. 
-        """
-        return json.dumps(self.curr_step.column_spreadsheet_code)
-
-    @property
-    def code_json(self) -> str:
-        """
-        This code json string is sent to the front-end and is what
-        ends up getting displayed in the codeblock. 
-        """
-        return json.dumps(transpile(self))
-
-    @property
-    def column_filters_json(self) -> str:
-        """
-        This column_filters list is used by the front end to display
-        the filtered icons in the UI
-        """
-        return json.dumps(self.curr_step.column_filters)
-    
-    @property
-    def column_type_json(self) -> str:
-        """
-        Returns a list of JSON objects that hold the type of each
-        data in each column.
-        """
-        return json.dumps(self.curr_step.column_type)
-
-    @property
-    def data_type_in_mito(self) -> DataTypeInMito:
-        return get_data_type_in_mito(self.dfs)
-
-    @property
-    def user_email(self) -> str:
-        """
-        Returns the user_email from user.json
-        """
-        return get_user_field(UJ_USER_EMAIL)
-
-    @property
-    def received_tours(self) -> str:
-        """
-        Returns the tours that the user has received
-        """
-        return json.dumps(get_user_field(UJ_RECEIVED_TOURS))
-
-    @property
-    def step_summaries_list_json(self) -> str:
+    def step_summary_list(self) -> List:
         """
         Returns a json list of step summaries, not including
         the skipped steps
         """
-        step_summaries_list = []
+        step_summary_list = []
         step_indexes_to_skip = get_step_indexes_to_skip(self.steps)
         for index, step in enumerate(self.steps):
             if step.step_type == 'initialize':
-                step_summaries_list.append({
+                step_summary_list.append({
                     'step_id': step.step_id,
                     'step_idx': 0,
                     'step_type': step.step_type,
@@ -325,7 +247,7 @@ class StepsManager():
             if index in step_indexes_to_skip:
                 continue
 
-            step_summaries_list.append({
+            step_summary_list.append({
                 'step_id': step.step_id,
                 'step_idx': index,
                 'step_type': step.step_type,
@@ -336,7 +258,7 @@ class StepsManager():
                 )
             })
 
-        return json.dumps(step_summaries_list)
+        return step_summary_list
 
     def handle_edit_event(self, edit_event):
         """
