@@ -47,7 +47,7 @@ def get_valid_dataframe_name(existing_df_names: List[str], original_dataframe_na
     ])
 
     # A valid variable name cannot be empty, or start with a number
-    if dataframe_name[0].isdecimal() or len(dataframe_name) == 0:
+    if len(dataframe_name) == 0 or dataframe_name[0].isdecimal():
         return 'df_' + dataframe_name
 
     return get_first_unused_dataframe_name(existing_df_names, dataframe_name)
@@ -73,23 +73,64 @@ def get_valid_dataframe_names(existing_df_names: List[str], original_df_names: L
 
 def dfs_to_json(
         dfs: List[pd.DataFrame],
+        df_names: List[str],
+        df_sources: List[str],
+        column_spreadsheet_code_array: List[Dict[str, str]],
+        column_filters_array: List[Dict[str, Any]],
+        column_mito_types_array: List[Dict[str, str]],
         column_ids: ColumnIDMap
     ) -> str:
     return json.dumps([
-        df_to_json_dumpsable(df, column_ids.column_header_to_column_id[sheet_index]) 
+        df_to_json_dumpsable(
+            df, 
+            df_names[sheet_index],
+            df_sources[sheet_index],
+            column_spreadsheet_code_array[sheet_index],
+            column_filters_array[sheet_index],
+            column_mito_types_array[sheet_index],
+            column_ids.column_header_to_column_id[sheet_index],
+            column_ids.column_id_to_column_header[sheet_index]
+        ) 
         for sheet_index, df in enumerate(dfs)
     ])
 
 
 def df_to_json_dumpsable(
         df: pd.DataFrame,
+        df_name: str,
+        df_source: str,
+        column_spreadsheet_code: Dict[str, str],
+        column_filters: Dict[str, Any],
+        column_mito_types: Dict[str, str],
         column_headers_to_column_ids: Dict[Any, str],
+        column_ids_to_column_headers: Dict[str, Any],
         length_restriction=True # Used if we want to only return the first 2k rows of the dataframe
     ):
     """
     Returns a dataframe represented in a way that can be turned into a 
-    JSON object with json.dumps
+    JSON object with json.dumps.
+
+    Should follow the format:
+    {
+        dfName: string;
+        dfSource: DFSource;
+        numRows: number,
+        numColumns: number,
+        data: {
+            columnID: string;
+            columnHeader: (string | number);
+            columnData: (string | number)[];
+        }[];
+        columnIDsMap: ColumnIDsMap;
+        columnSpreadsheetCodeMap: Record<string, string>;
+        columnFiltersMap: ColumnFilterMap;
+        columnMitoTypeMap: ColumnMitoTypeMap
+        index: (string | number)[];
+    }
     """
+
+    (num_rows, num_columns) = df.shape 
+
     # First, if the length_restriction is True, we get the head, and then 
     # we make a copy (as we modify the df below)
     # NOTE: we get the head first for efficiency reasons.
@@ -125,17 +166,36 @@ def df_to_json_dumpsable(
         for idx, e in enumerate(d):
             if e is None:
                 d[idx] = 'NaN'
-    
-    # Then, turn the column headers to their IDs, so that
-    # the frontend can access them properly
-    json_obj['columns'] = [
-        column_headers_to_column_ids[column_header] for column_header in json_obj['columns']
-    ]
-    # And instead of having them be called columns, be clear they are columnIDs
-    json_obj['columnIDs'] = json_obj['columns']
-    del json_obj['columns']
 
-    return json_obj
+    final_data = []
+    for column_index, column_header in enumerate(json_obj['columns']):
+        column_final_data = {
+            'columnID': column_headers_to_column_ids[column_header],
+            'columnHeader': column_header,
+            'columnData': []
+        }
+        for row in json_obj['data']:
+            column_final_data['columnData'].append(row[column_index])
+        
+        final_data.append(column_final_data)     
+    
+    return {
+        "dfName": df_name,
+        "dfSource": df_source,
+        'numRows': num_rows,
+        'numColumns': num_columns,
+        'data': final_data,
+        # NOTE: We make sure that all the maps are in the correct order, so things are easy on the
+        # front-end and we don't have to worry about sorting
+        'columnIDsMap': {
+            column_headers_to_column_ids[column_header]: column_header
+            for column_header in df.keys()
+        },
+        'columnSpreadsheetCodeMap': column_spreadsheet_code,
+        'columnFiltersMap': column_filters,
+        'columnMitoTypeMap': column_mito_types,
+        'index': json_obj['index']
+    }
 
 
 def get_random_id():
